GS = 'Gold standard'
SUPERKINGDOM = 'superkingdom'
PHYLUM = 'phylum'
CLASS = 'class'
ORDER = 'order'
FAMILY = 'family'
GENUS = 'genus'
SPECIES = 'species'
STRAIN = 'strain'
PHYLUM_SPECIES = [PHYLUM, CLASS, ORDER, FAMILY, GENUS, SPECIES]
ALL_RANKS = [SUPERKINGDOM, PHYLUM, CLASS, ORDER, FAMILY, GENUS, SPECIES, STRAIN]

UNIFRAC = 'Weighted UniFrac error'
UNW_UNIFRAC = 'Unweighted UniFrac error'
L1NORM = 'L1 norm error'
PRECISION = 'Purity (precision)'
RECALL = 'Completeness (recall)'
F1_SCORE = 'F1 score'
TP = "True positives"
FP = 'False positives'
FN = "False negatives"
OTUS = "Taxon counts"
JACCARD = "Jaccard index"
SHANNON_DIVERSITY = 'Shannon diversity'
SHANNON_EQUIT = 'Shannon equitability'
BRAY_CURTIS = 'Bray-Curtis distance'
ALL_METRICS = [UNIFRAC, UNW_UNIFRAC, L1NORM, PRECISION, RECALL, F1_SCORE, TP, FP, FN, JACCARD, SHANNON_DIVERSITY, SHANNON_EQUIT, BRAY_CURTIS]

TOOLTIP_UNIFRAC = 'tree-based measure of similarity between the true and predicted abundances at all taxonomic ranks ranging from 0 (high similarity) to 16 (low similarity).'
TOOLTIP_UNW_UNIFRAC = 'similar to the weighted UniFrac error, but measuring how well a profiler correctly identified the presence and absence of taxa instead of their relative abundances. The maximal value is the product of the number of taxonomic ranks minus 1 and the number of true taxa at all taxonomic ranks.'
TOOLTIP_L1NORM = 'sum of the absolute differences between the true and predicted abundances at the selected taxonomic rank ranging from 0 (perfect reconstruction of the relative abundances) to 2 (totally incorrect reconstruction).'
TOOLTIP_PRECISION = 'ratio of taxa correctly predicted as present and all predicted taxa at the selected taxonomic rank. It ranges from 0 (worst) to 1 (best).'
TOOLTIP_RECALL = 'ratio of taxa correctly predicted as present and all taxa present at the selected taxonomic rank. It ranges from 0 (worst) to 1 (best).'
TOOLTIP_F1_SCORE = 'harmonic average of completeness and purity at the selected taxonomic rank. It ranges from 0 (worst) to 1 (best).'
TOOLTIP_TP = "number of taxa correctly predicted as present at the selected taxonomic rank."
TOOLTIP_FP = 'number of taxa incorrectly predicted as present at the selected taxonomic rank.'
TOOLTIP_FN = "number of taxa incorrectly predicted as absent at the selected taxonomic rank."
TOOLTIP_OTUS = "number of predicted taxa at the selected taxonomic rank."
TOOLTIP_JACCARD = "indicator of similarity between the sets of true and predicted taxa at the seletec taxonomic rank computed as the ratio of the number of taxa in the intersection of these sets to the number of taxa in their union. It ranges from 0 (complete dissimilarity) to 1 (complete overlap)."
TOOLTIP_SHANNON_DIVERSITY = 'ranges from 0 to the natural logarithm of the number of predicted taxa, where the latter represents maximal possible diversity with all taxa being evenly represented.'
TOOLTIP_SHANNON_EQUIT = 'normalized form of Shannon diversity index ranging from 0 to 1 with 1 indicating complete evenness.'
TOOLTIP_BRAY_CURTIS = 'normalized form of the L1 norm ranging from 0 to 1.'



# This will return the EMDUnifrac distance only
def EMDUnifrac_weighted(Tint, lint, nodes_in_order, P, Q):
    '''
    (Z, diffab) = EMDUnifrac_weighted(Tint, lint, nodes_in_order, P, Q)
    This function takes the ancestor dictionary Tint, the lengths dictionary lint, the basis nodes_in_order
    and two probability vectors P and Q (typically P = envs_prob_dict[samples[i]], Q = envs_prob_dict[samples[j]]).
    Returns the weighted Unifrac distance Z and the flow F. The flow F is a dictionary with keys of the form (i,j) where
    F[(i,j)] == num means that in the calculation of the Unifrac distance, a total mass of num was moved from the node
    nodes_in_order[i] to the node nodes_in_order[j].
    '''
    num_nodes = len(nodes_in_order)
    Z = 0
    diffab = dict()
    partial_sums = P - Q
    for i in range(num_nodes - 1):
        val = partial_sums[i]
        partial_sums[Tint[i]] += val
        if val != 0:
            diffab[(i, Tint[i])] = lint[i, Tint[i]] * val  # Captures diffab
        Z += lint[i, Tint[i]] * abs(val)
    return (Z, diffab)


# This will return the EMDUnifrac distance only
def EMDUnifrac_unweighted(Tint, lint, nodes_in_order, P, Q):
    '''
    (Z, diffab) = EMDUnifrac_unweighted(Tint, lint, nodes_in_order, P, Q)
    This function takes the ancestor dictionary Tint, the lengths dictionary lint, the basis nodes_in_order
    and two probability vectors P and Q (typically P = envs_prob_dict[samples[i]], Q = envs_prob_dict[samples[j]]).
    Returns the unweighted Unifrac distance Z and the flow F. The flow F is a dictionary with keys of the form (i,j) where
    F[(i,j)] == 1 means that in the calculation of the Unifrac distance, a total mass of 1 was moved from the node
    nodes_in_order[i] to the node nodes_in_order[j].
    '''
    num_nodes = len(nodes_in_order)
    Z = 0
    diffab = dict()
    for i in range(num_nodes):
        if P[i] > 0:
            P[i] = 1
        if Q[i] > 0:
            Q[i] = 1
    partial_sums = P - Q
    for i in range(num_nodes - 1):
        val = partial_sums[i]
        partial_sums[Tint[i]] += val
        if val != 0:
            diffab[(i, Tint[i])] = lint[i, Tint[i]] * val  # Captures diffab
        Z += lint[i, Tint[i]] * abs(val)
    return Z, diffab



import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from io import StringIO

# Metaphlan 2.2.0 & 12.50h & (1.66GB) & 2.13h & (1.44GB)\\
# FOCUS CAMI & 13.45h & (4.99GB) & 2.34h & (4.99GB)\\
# Quikr & 16.32h & (26.12GB) & 2.12h & (24.12GB)\\
# mOTU 1.1 & 19.83h & (1.19GB) & 3.82h & (1.19GB)\\
# CommonKmers & 76.82h & (26.03GB) & 11.28h & (24.16GB)\\
# MetaPhyler 1.25 & 119.51h & (2.56GB) & 31.16h & (6.85GB)\\
# TIPP 2.0.0 & 151.01h & (67.51GB) & 32.01h & (81.53GB)\\

# s = StringIO("""     time     memory
# Metaphlan    12.50   1.66
# FOCUS        13.45   4.99
# Quikr        16.32  26.12
# mOTU         19.83   1.19
# CommonKmers  76.82  26.03
# MetaPhyler  119.51   2.56
# TIPP        151.01   67.5""")

s = StringIO("""     time     memory
Metaphlan   2.13     1.44
FOCUS       2.34     4.99
Quikr       2.12    24.12
mOTU        3.82     1.19
CommonKmers 11.28   24.16
MetaPhyler  31.16    6.85
TIPP        32.01   81.53""")


def plot_time_memory(time, memory, tool):
    # print(dict(zip(range(len(tool)), tool)))
    # exit()
    # xx = map(tool, range(len(tool)))
    # print(dict(xx))
    # exit()


    time_label = 'Time (hours)'
    memory_label = 'Memory (GB)'
    df = pd.DataFrame({'tool': tool, time_label: time, memory_label: memory}, columns=[time_label, memory_label], index=tool).sort_values(by=[time_label])
    # df.set_index('tool', inplace=True)
    # df.sort_values(by=[time_label], inplace=True)
    # df = pd.read_csv(s, index_col=0, delimiter=' ', skipinitialspace=True)
    # df.set_index(["a", "b", "c", "d", "e", "f", "g"], inplace=True)
    print(df.index.tolist())
    print(df.columns)
    print(df)
    exit()
    df.columns = ['Time (hours)', 'Memory (GB)']

    ax = df.plot(kind='bar', secondary_y='Memory (GB)', mark_right=False, zorder=20)
    ax.grid(which='major', linestyle='-', linewidth='0.5', color='lightgrey', zorder=0)
    ax.set_ylabel('Time (hours)')
    plt.setp(ax.get_xticklabels(), fontsize=9, rotation=15)

    ax2 = ax.twinx()
    ax2.set_ylabel('Memory (GB)', labelpad=23)
    ax2.yaxis.set_ticks([])

    plt.savefig('test.pdf', dpi=100, format='pdf', bbox_inches='tight')


plot_time_memory([11.28, 2.13, 32.01], [24.16, 1.44, 81.53], ['a', 'b', 'c'])



#!/usr/bin/env python

from src.utils import constants as c
import pandas as pd
import logging


def get_user_ranks_list(ranks):
    rank_high_low = [x.strip() for x in ranks.split(',')]
    if len(rank_high_low) != 2 or rank_high_low[0] not in c.ALL_RANKS or rank_high_low[1] not in c.ALL_RANKS:
        logging.getLogger('opal').warning('Invalid ranks provided with option --ranks. Default will be used.')
        return c.ALL_RANKS[:7]
    index1 = c.ALL_RANKS.index(rank_high_low[0])
    index2 = c.ALL_RANKS.index(rank_high_low[1])
    if index1 < index2:
        return c.ALL_RANKS[index1:index2 + 1]
    else:
        return c.ALL_RANKS[index2:index1 + 1]


def highscore_table(metrics, ranks):
    """Compile a ranking table like Figure 3c of CAMI publication.
    
    Note that Figure 3c took into account mean scores for all samples of one of the three
    complexity levels, i.e. 1 for low, 2 for medium, 5 for high.
    Here, I assume that we might be able to do that later, but for now I set "complexity"
    to "dummy".
    
    Parameters
    ----------
    metrics : pd.DataFrame
        Information about metrics of tool performance.
        Must contain columns: metric, rank, tool, value
    useranks : [str]
        Old default (CAMI 1): 'phylum', 'class', 'order', 'family', 'genus'
        Which ranks should be considered for rank dependent metrics.
        Here we decided to exclude e.g. species, because most profilers
        fail at that rank and we don't want to emphasize on this rank.
    Returns
    -------
    Pandas.DataFrame holding a high scoring table as in Figure 3c.
    """
    if ranks:
        useranks = get_user_ranks_list(ranks)
    else:
        useranks = c.ALL_RANKS[:7]

    pd_metrics = metrics.copy()
    pd_metrics.loc[pd_metrics[pd.isnull(pd_metrics['rank'])].index, 'rank'] = 'rank independent'

    sort_ascendingly = {c.L1NORM: True, c.UNIFRAC: True,
                        c.RECALL: False, c.PRECISION: False}

    # get rid of gold standard
    pd_metrics = pd_metrics[pd_metrics['tool'] != c.GS]

    # collecting rank scores
    posresults = []
    for (metric, sample, rank), g in pd_metrics.groupby(['metric', 'sample', 'rank']):
        if metric in sort_ascendingly:
            if ((rank in useranks) and (metric != c.UNIFRAC)) or ((rank == 'rank independent') and (metric == c.UNIFRAC)):
                res = g.groupby('tool').sum()
                res['position'] = res['value'].rank(method='min', ascending=sort_ascendingly[metric]) - 1
                res['metric'] = metric
                res['sample'] = sample
                res['rank'] = rank
                posresults.append(res)
    posresults = pd.concat(posresults)

    return posresults.groupby(['metric', 'tool'])['position'].sum().to_frame(), useranks

    # reformat like Figure 3c
    os = []
    for metric, g in posresults.groupby('metric'):
        highscore = g.groupby('tool')['position'].sum().sort_values()
        os.append(pd.DataFrame(["%s (%i)" % (idx, pos) for idx, pos in highscore.iteritems()], columns=[metric]))

    # return reformatted table
    return pd.concat(os, axis=1)  #.T.loc[['Recall', 'Precision', 'L1 norm error', 'Unweighted Unifrac error'],:]


#!/usr/bin/env python3

import os
import sys
import errno
import argparse
import os.path
import pandas as pd
import numpy as np
import logging
from src import l1norm as l1
from src import binary_metrics as bm
from src import unifrac_distance as uf
from src import shannon as sh
from src import braycurtis as bc
from src import rankings as rk
from src import html_opal as html
from src import plots as pl
from src.utils import load_data
from src.utils import ProfilingTools as PF
from src.utils import constants as c
from version import __version__


def make_sure_path_exists(path):
    try:
        os.makedirs(path)
    except OSError as exception:
        if exception.errno != errno.EEXIST:
            raise


def get_labels(labels, profiles_files):
    if labels:
        labels_list = [x.strip() for x in labels.split(',')]
        if len(labels_list) != len(profiles_files):
            logging.getLogger('opal').critical('The number of labels does not match the number of files of profiles. Please check parameter -l, --labels.')
            exit(1)
        return labels_list
    tool_id = []
    for profile_file in profiles_files:
        tool_id.append(profile_file.split('/')[-1])
    return tool_id


def get_time_memory(time, memory, profiles_files):
    time_list = []
    memory_list = []
    if time:
        time_list = [float(x.strip()) for x in time.split(',')]
        if len(time_list) != len(profiles_files):
            logging.getLogger('opal').critical('The number of running times does not match the number of files of profiles. Please check parameter --time.')
            exit(1)
    if memory:
        memory_list = [float(x.strip()) for x in memory.split(',')]
        if len(memory_list) != len(profiles_files):
            logging.getLogger('opal').critical('The number of memory usages does not match the number of files of profiles. Please check parameter --memory.')
            exit(1)
    return time_list, memory_list


def compute_binary_metrics(query_profile, query_truth):
    all_metrics = bm.compute_tree_metrics(query_profile, query_truth)
    return all_metrics


def print_by_rank(output_dir, labels, pd_metrics):
    make_sure_path_exists(os.path.join(output_dir, "by_rank"))
    for rank in c.ALL_RANKS:
        # define ordering of rows, which is given my order of tool labels
        order_rows = labels
        # define ordering of columns, hard coded
        order_columns = [c.UNIFRAC, c.UNW_UNIFRAC, c.L1NORM, c.RECALL, c.PRECISION, c.F1_SCORE, c.TP, c.FP, c.FN, c.OTUS, c.JACCARD, c.SHANNON_DIVERSITY, c.SHANNON_EQUIT, c.BRAY_CURTIS]
        # subset to those information that either belong to the given rank or are rank independent, i.e. are unifrac values
        table = pd_metrics[(pd_metrics['rank'] == rank) | (pd_metrics['metric'].isin([c.UNIFRAC, c.UNW_UNIFRAC]))]
        # reformat the table with a pivot_table
        table = table.pivot_table(index=['tool', 'sample'], columns='metric', values='value')
        # select only tools in labels and get rid of gold standard
        table = table.loc[pd.IndexSlice[order_rows,:], order_columns]
        # define categorical column for ordering rows by tools
        table['tool_cat'] = pd.Categorical(table.index.get_level_values('tool'), categories=order_rows, ordered=True)
        # order table
        table = table.sort_values('tool_cat')
        table = table.loc[:, order_columns]
        # replace np.NaN with string "na" and write resulting table into a file
        table.fillna('na').to_csv(os.path.join(output_dir, "by_rank", rank + ".tsv"), sep='\t')


def print_by_tool(output_dir, pd_metrics):
    make_sure_path_exists(os.path.join(output_dir, "by_tool"))
    # define ordering of columns, hard coded
    order_columns = [c.UNIFRAC, c.UNW_UNIFRAC, c.L1NORM, c.RECALL, c.PRECISION, c.F1_SCORE, c.TP, c.FP, c.FN, c.OTUS, c.JACCARD, c.SHANNON_DIVERSITY, c.SHANNON_EQUIT, c.BRAY_CURTIS]
    for toolname, pd_metrics_tool in pd_metrics.groupby('tool'):
        if toolname == c.GS:
            continue
        table = pd_metrics_tool.pivot_table(index=['rank', 'sample'], columns='metric', values='value')
        # little hack to carry unifrac over to every rank
        for unifrac_col in order_columns[:2]:
            table[unifrac_col] = pd_metrics_tool[pd_metrics_tool['metric'] == unifrac_col]['value'].values[0]
        # order table
        table['rank_cat'] = pd.Categorical(table.index.get_level_values('rank'), categories=c.ALL_RANKS, ordered=True)
        table = table.sort_values('rank_cat')
        table = table.loc[:, order_columns]
        # replace np.NaN with string "na" and write resulting table into a file
        table.fillna('na').to_csv(os.path.join(output_dir, "by_tool", toolname + ".tsv"), sep='\t')


def compute_metrics(sample_metadata, profile, gs_pf_profile, gs_rank_to_taxid_to_percentage, rank_to_taxid_to_percentage):
    # Unifrac
    if isinstance(profile, PF.Profile):
        pf_profile = profile
    else:
        pf_profile = PF.Profile(sample_metadata=sample_metadata, profile=profile)
    unifrac = uf.compute_unifrac(gs_pf_profile, pf_profile)

    # Shannon
    shannon = sh.compute_shannon_index(rank_to_taxid_to_percentage)

    # L1 Norm
    l1norm = l1.compute_l1norm(gs_rank_to_taxid_to_percentage, rank_to_taxid_to_percentage)

    # Binary metrics
    binary_metrics = compute_binary_metrics(rank_to_taxid_to_percentage, gs_rank_to_taxid_to_percentage)

    # Bray-Curtis
    braycurtis = bc.braycurtis(gs_rank_to_taxid_to_percentage, rank_to_taxid_to_percentage)

    return unifrac, shannon, l1norm, binary_metrics, braycurtis


def load_profiles(gold_standard_file, profiles_files, no_normalization):
    normalize = False if no_normalization else True

    gs_samples_list = load_data.open_profile(gold_standard_file, normalize)
    sample_ids_list = []
    for sample in gs_samples_list:
        sample_id, sample_metadata, profile = sample
        sample_ids_list.append(sample_id)

    profiles_list_to_samples_list = []
    for profile_file in profiles_files:
        profiles_list_to_samples_list.append(load_data.open_profile(profile_file, normalize))

    return sample_ids_list, gs_samples_list, profiles_list_to_samples_list


def evaluate(gs_samples_list, profiles_list_to_samples_list, labels):
    gs_id_to_rank_to_taxid_to_percentage = {}
    gs_id_to_pf_profile = {}
    pd_metrics = pd.DataFrame()

    for sample in gs_samples_list:
        sample_id, sample_metadata, profile = sample
        gs_id_to_rank_to_taxid_to_percentage[sample_id] = load_data.get_rank_to_taxid_to_percentage(profile)
        gs_id_to_pf_profile[sample_id] = PF.Profile(sample_metadata=sample_metadata, profile=profile)
        unifrac, shannon, l1norm, binary_metrics, braycurtis = compute_metrics(sample_metadata,
                                                                               gs_id_to_pf_profile[sample_id],
                                                                               gs_id_to_pf_profile[sample_id],
                                                                               gs_id_to_rank_to_taxid_to_percentage[sample_id],
                                                                               gs_id_to_rank_to_taxid_to_percentage[sample_id])
        pd_metrics = pd.concat([pd_metrics, reformat_pandas(sample_id, c.GS, braycurtis, shannon, binary_metrics, l1norm, unifrac)], ignore_index=True)

    one_profile_assessed = False
    for samples_list, label in zip(profiles_list_to_samples_list, labels):
        for sample in samples_list:
            sample_id, sample_metadata, profile = sample

            # match the sample id of the gold standard and the predictions
            if sample_id in gs_id_to_rank_to_taxid_to_percentage:
                gs_rank_to_taxid_to_percentage = gs_id_to_rank_to_taxid_to_percentage[sample_id]
                gs_pf_profile = gs_id_to_pf_profile[sample_id]
            else:
                sys.stderr.write("Skipping assessment of {} for sample {}. Make sure the SampleID of the gold standard and the profile are identical.\n".format(label, sample_id))
                continue

            rank_to_taxid_to_percentage = load_data.get_rank_to_taxid_to_percentage(profile)

            unifrac, shannon, l1norm, binary_metrics, braycurtis = compute_metrics(sample_metadata, profile, gs_pf_profile,
                                                                                   gs_rank_to_taxid_to_percentage,
                                                                                   rank_to_taxid_to_percentage)
            pd_metrics = pd.concat([pd_metrics, reformat_pandas(sample_id, label, braycurtis, shannon, binary_metrics, l1norm, unifrac)], ignore_index=True)
            one_profile_assessed = True

    if not one_profile_assessed:
        logging.getLogger('opal').critical("No profile could be evaluated.")
        exit(1)

    return pd_metrics


def reformat_pandas(sample_id, label, braycurtis, shannon, binary_metrics, l1norm, unifrac):
    """Reformats metrics data into one unified pandas DataFrame.

    Parameters
    ----------
    sample_id : str
    label : str
        str for tool name.
    braycurtis : float
    shannon : ?
    binary_metrics : ?
    l1norm : float
    unifrac : ?

    Returns
    -------
    Pandas.DataFrame with following columns: metric, rank, tool, value
    """
    # convert Unifrac
    pd_unifrac = pd.DataFrame(index=[sample_id], data=[unifrac], columns=[c.UNIFRAC, c.UNW_UNIFRAC]).stack().reset_index()
    pd_unifrac.columns = ['sample', 'metric', 'value']
    pd_unifrac['rank'] = np.nan
    pd_unifrac['tool'] = label

    # convert Shannon
    pd_shannon = pd.DataFrame([shannon[rank].get_pretty_dict() for rank in shannon.keys()]).set_index('rank').stack().reset_index().rename(columns={'level_1': 'metric', 0: 'value'})
    pd_shannon['metric'].replace(['diversity', 'equitability'], [c.SHANNON_DIVERSITY, c.SHANNON_EQUIT], inplace=True)
    pd_shannon['sample'] = sample_id
    pd_shannon['tool'] = label

    # convert L1 Norm
    pd_l1norm = pd.DataFrame(index=[sample_id], data=l1norm).stack().reset_index()
    pd_l1norm.columns = ['sample', 'rank', 'value']
    pd_l1norm['tool'] = label
    pd_l1norm['metric'] = c.L1NORM

    # convert Binary metrics
    pd_binary_metrics = pd.DataFrame([binary_metrics[rank].get_pretty_dict() for rank in binary_metrics.keys()]).set_index('rank').stack().reset_index().rename(columns={'level_1': 'metric', 0: 'value'})
    pd_binary_metrics['metric'].replace(['fp',
                                         'tp',
                                         "fn",
                                         "jaccard",
                                         "precision",
                                         "recall",
                                         "f1",
                                         "otus"],
                                        [c.FP,
                                         c.TP,
                                         c.FN,
                                         c.JACCARD,
                                         c.PRECISION,
                                         c.RECALL,
                                         c.F1_SCORE,
                                         c.OTUS], inplace=True)
    pd_binary_metrics['sample'] = sample_id
    pd_binary_metrics['tool'] = label

    # convert Bray-Curtis
    pd_braycurtis = pd.DataFrame(index=[sample_id], data=braycurtis).stack().reset_index()
    pd_braycurtis.columns = ['sample', 'rank', 'value']
    pd_braycurtis['tool'] = label
    pd_braycurtis['metric'] = c.BRAY_CURTIS

    return pd.concat([pd_braycurtis, pd_shannon, pd_binary_metrics, pd_l1norm, pd_unifrac], ignore_index=True, sort=False)


def create_output_directories(output_dir, labels):
    make_sure_path_exists(os.path.join(output_dir, 'gold_standard'))
    for label in labels:
        make_sure_path_exists(os.path.join(output_dir, "by_tool", label))


def concat_pd(labels, metric, values, pd_metrics):
    df = pd.DataFrame({'tool': labels, 'value': values})
    df['sample'] = np.nan
    df['metric'] = metric
    df['rank'] = np.nan
    return pd.concat([pd_metrics, df], ignore_index=True, sort=False)


def concat_time_memory(labels, time_list, memory_list, pd_metrics):
    if time_list:
        pd_metrics = concat_pd(labels, 'time', time_list, pd_metrics)
    if memory_list:
        pd_metrics = concat_pd(labels, 'memory', memory_list, pd_metrics)
    return pd_metrics


def get_logger(output_dir, silent):
    logger = logging.getLogger('opal')
    logger.setLevel(logging.INFO)
    formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
    logging_fh = logging.FileHandler(os.path.join(output_dir, 'log.txt'))
    logging_fh.setFormatter(formatter)
    logger.addHandler(logging_fh)

    if not silent:
        logging_stdout = logging.StreamHandler(sys.stdout)
        logging_stdout.setFormatter(formatter)
        logger.addHandler(logging_stdout)
    return logger


def main():
    parser = argparse.ArgumentParser(description='OPAL: Open-community Profiling Assessment tooL', add_help=False)
    group1 = parser.add_argument_group('required arguments')
    group1.add_argument('profiles_files', nargs='+', help='Files of profiles')
    group1.add_argument('-g', '--gold_standard_file', help='Gold standard file', required=True)
    group1.add_argument('-o', '--output_dir', help='Directory to write the results to', required=True)
    group2 = parser.add_argument_group('optional arguments')
    group2.add_argument('-n', '--no_normalization', help='Do not normalize samples', action='store_true')
    group2.add_argument('-p', '--plot_abundances', help='Plot abundances in the gold standard (can take some minutes)', action='store_true')
    group2.add_argument('-l', '--labels', help='Comma-separated profiles names', required=False)
    group2.add_argument('-t', '--time', help='Comma-separated runtimes in hours', required=False)
    group2.add_argument('-m', '--memory', help='Comma-separated memory usages in gigabytes', required=False)
    group2.add_argument('-d', '--desc', help='Description for HTML page', required=False)
    group2.add_argument('-r', '--ranks', help='Highest and lowest taxonomic ranks to consider in performance rankings, comma-separated. Valid ranks: superkingdom, phylum, class, order, family, genus, species, strain (default:superkingdom,species)', required=False)
    group2.add_argument('--silent', help='Silent mode', action='store_true')
    group2.add_argument('-v', '--version', action='version', version='%(prog)s ' + __version__)
    group2.add_argument('-h', '--help', action='help', help='Show this help message and exit')
    args = parser.parse_args()
    output_dir = os.path.abspath(args.output_dir)
    make_sure_path_exists(output_dir)
    labels = get_labels(args.labels, args.profiles_files)

    create_output_directories(output_dir, labels)

    logger = get_logger(args.output_dir, args.silent)

    logger.info('Loading profiles...')
    sample_ids_list, gs_samples_list, profiles_list_to_samples_list = load_profiles(args.gold_standard_file,
                                                                                    args.profiles_files,
                                                                                    args.no_normalization)
    logger.info('done')

    plots_list = []
    if args.plot_abundances:
        logger.info('Plotting gold standard abundances...')
        plots_list += pl.plot_samples_hist(gs_samples_list, sample_ids_list, output_dir)
        logger.info('done')

    logger.info('Computing metrics...')
    pd_metrics = evaluate(gs_samples_list,
                          profiles_list_to_samples_list,
                          labels)
    time_list, memory_list = get_time_memory(args.time, args.memory, args.profiles_files)
    if time_list or memory_list:
        pd_metrics = concat_time_memory(labels, time_list, memory_list, pd_metrics)
    logger.info('done')

    logger.info('Saving computed metrics...')
    pd_metrics[['tool', 'rank', 'metric', 'sample', 'value']].fillna('na').to_csv(os.path.join(output_dir, 'results.tsv'), sep='\t', index=False)
    print_by_tool(output_dir, pd_metrics)
    print_by_rank(output_dir, labels, pd_metrics)
    logger.info('done')

    logger.info('Creating beta diversity plots...')
    plots_list += pl.plot_beta_diversity(gs_samples_list, profiles_list_to_samples_list, sample_ids_list, labels, output_dir)
    logger.info('done')

    logger.info('Creating rarefaction curves...')
    plots_list += pl.plot_rarefaction_curves(gs_samples_list, output_dir)
    plots_list += pl.plot_rarefaction_curves(gs_samples_list, output_dir, log_scale=True)
    logger.info('done')

    logger.info('Creating spider plots...')
    plots_list += pl.plot_all(pd_metrics, labels, output_dir)
    logger.info('done')

    logger.info('Computing rankings...')
    pd_rankings, ranks_scored = rk.highscore_table(pd_metrics, args.ranks)
    logger.info('done')

    if time_list or memory_list:
        logger.info('Plotting computing efficiency...')
        plots_list += pl.plot_time_memory(time_list, memory_list, labels, output_dir)
        logger.info('done')

    logger.info('Creating HTML page...')
    html.create_html(pd_rankings, ranks_scored, pd_metrics, labels, sample_ids_list, plots_list, output_dir, args.desc)
    logger.info('done')

    logger.info('OPAL finished successfully. All results have been saved to {}'.format(output_dir))


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import sys
import argparse
from src.utils import load_data
from src.utils import constants as c


def compute_l1norm(f1_rank_to_taxid_to_percentage, f2_rank_to_taxid_to_percentage):
    l1norm = {}
    for rank in f1_rank_to_taxid_to_percentage:
        if rank not in f2_rank_to_taxid_to_percentage:
            continue
        l1norm[rank] = .0
        taxa_union = set(f1_rank_to_taxid_to_percentage[rank].keys()).union(f2_rank_to_taxid_to_percentage[rank].keys())
        for taxid in taxa_union:
            percentage1 = f1_rank_to_taxid_to_percentage[rank][taxid] if taxid in f1_rank_to_taxid_to_percentage[rank] else .0
            percentage2 = f2_rank_to_taxid_to_percentage[rank][taxid] if taxid in f2_rank_to_taxid_to_percentage[rank] else .0
            l1norm[rank] += abs(percentage1 - percentage2)
        l1norm[rank] = l1norm[rank] / 100.0
    return l1norm


def print_l1norm(l1norm, stream=sys.stdout):
    for rank in l1norm:
        stream.write("{}\t{}\n".format(rank, l1norm[rank]))


def print_list_l1norm(l1norm_list, labels, stream=sys.stdout):
    labels_iterator = iter(labels)
    stream.write("tool\tsuperkingdom\tphylum\tclass\torder\tfamily\tgenus\tspecies\tstrain\n")
    for l1norm in l1norm_list:
        stream.write("{}\t".format(next(labels_iterator)))
        stream.write("\t".join([format(l1norm[c.SUPERKINGDOM], '.3f') if c.SUPERKINGDOM in l1norm else "na",
                      format(l1norm[c.PHYLUM], '.3f') if c.PHYLUM in l1norm else "na",
                      format(l1norm[c.CLASS], '.3f') if c.CLASS in l1norm else "na",
                      format(l1norm[c.ORDER], '.3f') if c.ORDER in l1norm else "na",
                      format(l1norm[c.FAMILY], '.3f') if c.FAMILY in l1norm else "na",
                      format(l1norm[c.GENUS], '.3f') if c.GENUS in l1norm else "na",
                      format(l1norm[c.SPECIES], '.3f') if c.SPECIES in l1norm else "na",
                      format(l1norm[c.STRAIN], '.3f') if c.STRAIN in l1norm else "na"]))
        stream.write("\n")


def main():
    parser = argparse.ArgumentParser(description="Compute L1 Norm")
    parser.add_argument('-1', '--file1', help="Profile 1", required=True)
    parser.add_argument('-2', '--file2', help="Profile 2", required=True)
    args = parser.parse_args()
    f1_rank_to_taxid_to_percentage = load_data.open_profile(args.file1)
    f2_rank_to_taxid_to_percentage = load_data.open_profile(args.file2)
    l1norm = compute_l1norm(f1_rank_to_taxid_to_percentage, f2_rank_to_taxid_to_percentage)
    print_l1norm(l1norm)


if __name__ == "__main__":
    main()


from scipy.spatial import distance


def braycurtis(gs_rank_to_taxid_to_percentage, rank_to_taxid_to_percentage):
    rank_to_braycurtis = {}
    for rank in gs_rank_to_taxid_to_percentage:
        if rank not in rank_to_taxid_to_percentage:
            continue
        gs = []
        predictions = []
        taxa_union = set(gs_rank_to_taxid_to_percentage[rank].keys()).union(rank_to_taxid_to_percentage[rank].keys())
        for taxid in taxa_union:
            gs.append(gs_rank_to_taxid_to_percentage[rank][taxid] if taxid in gs_rank_to_taxid_to_percentage[rank] else .0)
            predictions.append(rank_to_taxid_to_percentage[rank][taxid] if taxid in rank_to_taxid_to_percentage[rank] else .0)
        if len(gs) > 0:
            rank_to_braycurtis[rank] = distance.braycurtis(gs, predictions)
    return rank_to_braycurtis



import numpy as np

import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from matplotlib.path import Path
from matplotlib.spines import Spine
from matplotlib.projections.polar import PolarAxes
from matplotlib.projections import register_projection


def radar_factory(num_vars, frame='circle'):
    """Create a radar chart with `num_vars` axes.

    This function creates a RadarAxes projection and registers it.

    Parameters
    ----------
    num_vars : int
        Number of variables for radar chart.
    frame : {'circle' | 'polygon'}
        Shape of frame surrounding axes.

    """
    # calculate evenly-spaced axis angles
    theta = np.linspace(0, 2*np.pi, num_vars, endpoint=False)
    # rotate theta such that the first axis is at the top
    theta += np.pi/2

    def draw_poly_patch(self):
        verts = unit_poly_verts(theta)
        return plt.Polygon(verts, closed=True, edgecolor='k')

    def draw_circle_patch(self):
        # unit circle centered on (0.5, 0.5)
        return plt.Circle((0.5, 0.5), 0.5)

    patch_dict = {'polygon': draw_poly_patch, 'circle': draw_circle_patch}
    if frame not in patch_dict:
        raise ValueError('unknown value for `frame`: %s' % frame)

    class RadarAxes(PolarAxes):

        name = 'radar'
        # use 1 line segment to connect specified points
        RESOLUTION = 1
        # define draw_frame method
        draw_patch = patch_dict[frame]

        def fill(self, *args, **kwargs):
            """Override fill so that line is closed by default"""
            closed = kwargs.pop('closed', True)
            return super(RadarAxes, self).fill(closed=closed, *args, **kwargs)

        def plot(self, *args, **kwargs):
            """Override plot so that line is closed by default"""
            lines = super(RadarAxes, self).plot(*args, **kwargs)
            for line in lines:
                self._close_line(line)

        def _close_line(self, line):
            x, y = line.get_data()
            # FIXME: markers at x[0], y[0] get doubled-up
            if x[0] != x[-1]:
                x = np.concatenate((x, [x[0]]))
                y = np.concatenate((y, [y[0]]))
                line.set_data(x, y)

        def set_varlabels(self, labels):
            self.set_thetagrids(np.degrees(theta), labels, fontsize='small')

        def _gen_axes_patch(self):
            return self.draw_patch()

        def _gen_axes_spines(self):
            if frame == 'circle':
                return PolarAxes._gen_axes_spines(self)
            # The following is a hack to get the spines (i.e. the axes frame)
            # to draw correctly for a polygon frame.

            # spine_type must be 'left', 'right', 'top', 'bottom', or `circle`.
            spine_type = 'circle'
            verts = unit_poly_verts(theta)
            # close off polygon by repeating first vertex
            verts.append(verts[0])
            path = Path(verts)

            spine = Spine(self, spine_type, path)
            spine.set_transform(self.transAxes)
            return {'polar': spine}

    register_projection(RadarAxes)
    return theta


def unit_poly_verts(theta):
    """Return vertices of polygon for subplot axes.

    This polygon is circumscribed by a unit circle centered at (0.5, 0.5)
    """
    x0, y0, r = [0.5] * 3
    verts = [(r*np.cos(t) + x0, r*np.sin(t) + y0) for t in theta]
    return verts



import pkg_resources, os, pkgutil
from setuptools import setup, find_packages
from version import __version__
import glob


def dependencies():
    file_ = pkg_resources.resource_filename(__name__, os.path.join('requirements', 'default.txt'))
    with open(file_, 'r') as f:
        return f.read().splitlines()

setup(
    name                 = 'cami-opal',
    version              = __version__,
    description          = 'OPAL: Open-community Profiling Assessment tooL',
    author               = 'CAMI',
    author_email         = 'contact@cami-challenge.org',
    url                  = 'http://cami-challenge.org',
    scripts              = glob.glob('*.py'),
    install_requires     = dependencies(),
    packages             = find_packages(),
    classifiers = [
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Intended Audience :: Science/Research',
        'Operating System :: POSIX'
    ]
)
